@@ -19,6 +19,9 @@
 #include "orcus/spreadsheet/import_interface.hpp"
 #include "orcus/measurement.hpp"
 
+#include <mdds/sorted_string_map.hpp>
+#include <mdds/global.hpp>
+
 #include <iostream>
 #include <fstream>
 #include <cstdlib>
@@ -246,6 +249,26 @@ void xlsx_shared_strings_context::characters(const pstring& str, bool transient)
 
 namespace {
 
+typedef mdds::sorted_string_map<spreadsheet::border_style_t> border_style_map;
+
+border_style_map::entry border_style_type_entries[] =
+{
+    {MDDS_ASCII("dashDot"), spreadsheet::border_style_t::dash_dot},
+    {MDDS_ASCII("dashDotDot"), spreadsheet::border_style_t::dash_dot_dot},
+    {MDDS_ASCII("dashed"), spreadsheet::border_style_t::dashed},
+    {MDDS_ASCII("dotted"), spreadsheet::border_style_t::dotted},
+    {MDDS_ASCII("double"), spreadsheet::border_style_t::double_border},
+    {MDDS_ASCII("hair"), spreadsheet::border_style_t::hair},
+    {MDDS_ASCII("medium"), spreadsheet::border_style_t::medium},
+    {MDDS_ASCII("mediumDashDot"), spreadsheet::border_style_t::medium_dash_dot},
+    {MDDS_ASCII("mediumDashDotDot"), spreadsheet::border_style_t::medium_dash_dot_dot},
+    {MDDS_ASCII("mediumDashed"), spreadsheet::border_style_t::medium_dashed},
+    {MDDS_ASCII("none"), spreadsheet::border_style_t::none},
+    {MDDS_ASCII("slantDashDot"), spreadsheet::border_style_t::slant_dash_dot},
+    {MDDS_ASCII("thick"), spreadsheet::border_style_t::thick},
+    {MDDS_ASCII("thin"), spreadsheet::border_style_t::thin}
+};
+
 class border_attr_parser : public unary_function<xml_token_attr_t, void>
 {
     spreadsheet::border_direction_t m_dir;
@@ -259,7 +282,11 @@ public:
         switch (attr.name)
         {
             case XML_style:
-                m_styles.set_border_style(m_dir, attr.value.get(), attr.value.size());
+            {
+                border_style_map type_map(border_style_type_entries, ORCUS_N_ELEMENTS(border_style_type_entries), spreadsheet::border_style_t::none);
+                  m_styles.set_border_style(m_dir,
+                          type_map.find(attr.value.get(), attr.value.size()));
+            }
             break;
         }
     }
@@ -426,8 +453,8 @@ class cell_alignment_attr_parser : public unary_function<xml_token_attr_t, void>
 
 public:
     cell_alignment_attr_parser() :
-        m_hor_align(spreadsheet::hor_alignment_unknown),
-        m_ver_align(spreadsheet::ver_alignment_bottom) // 'bottom' is the default if no vertical alignment is given.
+        m_hor_align(spreadsheet::hor_alignment_t::unknown),
+        m_ver_align(spreadsheet::ver_alignment_t::bottom) // 'bottom' is the default if no vertical alignment is given.
     {}
 
     void operator() (const xml_token_attr_t& attr)
@@ -437,21 +464,21 @@ public:
             case XML_horizontal:
             {
                 if (attr.value == "center")
-                    m_hor_align = spreadsheet::hor_alignment_center;
+                    m_hor_align = spreadsheet::hor_alignment_t::center;
                 else if (attr.value == "right")
-                    m_hor_align = spreadsheet::hor_alignment_right;
+                    m_hor_align = spreadsheet::hor_alignment_t::right;
                 else if (attr.value == "left")
-                    m_hor_align = spreadsheet::hor_alignment_left;
+                    m_hor_align = spreadsheet::hor_alignment_t::left;
             }
             break;
             case XML_vertical:
             {
                 if (attr.value == "top")
-                    m_ver_align = spreadsheet::ver_alignment_top;
+                    m_ver_align = spreadsheet::ver_alignment_t::top;
                 else if (attr.value == "center")
-                    m_ver_align = spreadsheet::ver_alignment_middle;
+                    m_ver_align = spreadsheet::ver_alignment_t::middle;
                 else if (attr.value == "bottom")
-                    m_ver_align = spreadsheet::ver_alignment_bottom;
+                    m_ver_align = spreadsheet::ver_alignment_t::bottom;
             }
             break;
             default:
@@ -475,7 +502,7 @@ public:
 xlsx_styles_context::xlsx_styles_context(session_context& session_cxt, const tokens& tokens, spreadsheet::iface::import_styles* styles) :
     xml_context_base(session_cxt, tokens),
     mp_styles(styles),
-    m_cur_border_dir(spreadsheet::border_direction_unknown),
+    m_cur_border_dir(spreadsheet::border_direction_t::unknown),
     m_cell_style_xf(false) {}
 
 xlsx_styles_context::~xlsx_styles_context() {}
@@ -536,13 +563,13 @@ void xlsx_styles_context::start_element(xmlns_id_t ns, xml_token_t name, const x
             xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
             pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_val)).get_value();
             if (ps == "double")
-                mp_styles->set_font_underline(spreadsheet::underline_double);
+                mp_styles->set_font_underline(spreadsheet::underline_t::double_line);
             else if (ps == "single")
-                mp_styles->set_font_underline(spreadsheet::underline_single);
+                mp_styles->set_font_underline(spreadsheet::underline_t::single_line);
             else if (ps == "singleAccounting")
-                mp_styles->set_font_underline(spreadsheet::underline_single_accounting);
+                mp_styles->set_font_underline(spreadsheet::underline_t::single_accounting);
             else if (ps == "doubleAccounting")
-                mp_styles->set_font_underline(spreadsheet::underline_double_accounting);
+                mp_styles->set_font_underline(spreadsheet::underline_t::double_accounting);
         }
         break;
         case XML_sz:
@@ -651,40 +678,40 @@ void xlsx_styles_context::start_element(xmlns_id_t ns, xml_token_t name, const x
         case XML_top:
         {
             xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
-            m_cur_border_dir = spreadsheet::border_top;
-            border_attr_parser func(spreadsheet::border_top, *mp_styles);
+            m_cur_border_dir = spreadsheet::border_direction_t::top;
+            border_attr_parser func(spreadsheet::border_direction_t::top, *mp_styles);
             for_each(attrs.begin(), attrs.end(), func);
         }
         break;
         case XML_bottom:
         {
             xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
-            m_cur_border_dir = spreadsheet::border_bottom;
-            border_attr_parser func(spreadsheet::border_bottom, *mp_styles);
+            m_cur_border_dir = spreadsheet::border_direction_t::bottom;
+            border_attr_parser func(spreadsheet::border_direction_t::bottom, *mp_styles);
             for_each(attrs.begin(), attrs.end(), func);
         }
         break;
         case XML_left:
         {
             xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
-            m_cur_border_dir = spreadsheet::border_left;
-            border_attr_parser func(spreadsheet::border_left, *mp_styles);
+            m_cur_border_dir = spreadsheet::border_direction_t::left;
+            border_attr_parser func(spreadsheet::border_direction_t::left, *mp_styles);
             for_each(attrs.begin(), attrs.end(), func);
         }
         break;
         case XML_right:
         {
             xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
-            m_cur_border_dir = spreadsheet::border_right;
-            border_attr_parser func(spreadsheet::border_right, *mp_styles);
+            m_cur_border_dir = spreadsheet::border_direction_t::right;
+            border_attr_parser func(spreadsheet::border_direction_t::right, *mp_styles);
             for_each(attrs.begin(), attrs.end(), func);
         }
         break;
         case XML_diagonal:
         {
             xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
-            m_cur_border_dir = spreadsheet::border_diagonal;
-            border_attr_parser func(spreadsheet::border_diagonal, *mp_styles);
+            m_cur_border_dir = spreadsheet::border_direction_t::diagonal;
+            border_attr_parser func(spreadsheet::border_direction_t::diagonal, *mp_styles);
             for_each(attrs.begin(), attrs.end(), func);
         }
         break;
