@@ -1,6 +1,6 @@
 /*************************************************************************
  *
- * Copyright (c) 2012-2014 Kohei Yoshida
+ * Copyright (c) 2012-2015 Kohei Yoshida
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -39,7 +39,6 @@
 #include <vector>
 
 #include <boost/ptr_container/ptr_vector.hpp>
-#include <boost/noncopyable.hpp>
 
 #define ARRAY_SIZE(x) sizeof(x)/sizeof(x[0])
 
@@ -90,10 +89,14 @@ struct date
 };
 
 template<typename T>
-class cell_pool : boost::noncopyable
+class cell_pool
 {
     boost::ptr_vector<T> m_pool;
 public:
+    cell_pool() = default;
+    cell_pool(const cell_pool&) = delete;
+    cell_pool& operator=(const cell_pool&) = delete;
+
     T* construct()
     {
         m_pool.push_back(new T);
@@ -106,6 +109,7 @@ public:
 class user_cell_pool : public cell_pool<user_cell>
 {
 public:
+
     user_cell* construct(double val)
     {
         user_cell* p = cell_pool<user_cell>::construct();
@@ -119,8 +123,8 @@ typedef mdds::mtv::managed_element_block<element_type_muser_block, muser_cell> m
 typedef mdds::mtv::default_element_block<element_type_fruit_block, my_fruit_type> fruit_block;
 typedef mdds::mtv::default_element_block<element_type_date_block, date> date_block;
 
-MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(user_cell, element_type_user_block, NULL, user_cell_block)
-MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(muser_cell, element_type_muser_block, NULL, muser_cell_block)
+MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(user_cell, element_type_user_block, nullptr, user_cell_block)
+MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(muser_cell, element_type_muser_block, nullptr, muser_cell_block)
 MDDS_MTV_DEFINE_ELEMENT_CALLBACKS(my_fruit_type, element_type_fruit_block, unknown_fruit, fruit_block)
 MDDS_MTV_DEFINE_ELEMENT_CALLBACKS(date, element_type_date_block, date(), date_block)
 
@@ -161,10 +165,10 @@ void mtv_test_types()
     assert(ct == mtv::element_type_boolean);
 
     // Custom cell type
-    user_cell* p = NULL;
+    user_cell* p = nullptr;
     ct = mtv_type::get_element_type(p);
     assert(ct == element_type_user_block && ct >= mtv::element_type_user_start);
-    ct = mtv_type::get_element_type(static_cast<muser_cell*>(NULL));
+    ct = mtv_type::get_element_type(static_cast<muser_cell*>(nullptr));
     assert(ct == element_type_muser_block && ct >= mtv::element_type_user_start);
     ct = mtv_fruit_type::get_element_type(unknown_fruit);
     assert(ct == element_type_fruit_block && ct >= mtv::element_type_user_start);
@@ -336,7 +340,7 @@ void mtv_test_basic()
         // Get empty value.
         mtv_type db(1);
         user_cell* p = db.get<user_cell*>(0);
-        assert(p == NULL);
+        assert(p == nullptr);
     }
 }
 
@@ -961,6 +965,36 @@ void mtv_test_managed_block()
         delete p1;
         delete p2;
     }
+
+    {
+        mtv_type db(5);
+
+        db.set(1, new muser_cell(1.1));
+        db.set(2, new muser_cell(1.2));
+        db.set(3, new muser_cell(1.3));
+
+        db.set(1, 2.1); // Don't leak the overwritten muser_cell instance.
+        db.set(2, 2.2); // ditto
+    }
+
+    {
+        mtv_type db(4);
+        db.set(0, new muser_cell(1.1));
+        db.set(1, new muser_cell(1.2));
+        db.set(2, new muser_cell(1.3));
+
+        db.set(2, 2.1); // Don't leak the overwritten muser_cell instance.
+        db.set(1, 2.0); // ditto
+    }
+
+    {
+        mtv_type db(8);
+        db.set(3, new muser_cell(1.1));
+        db.set(4, new muser_cell(1.2));
+        db.set(5, 1.3);
+
+        db.set(4, 2.2); // Overwrite muser_cell and don't leak.
+    }
 }
 
 void mtv_test_custom_block_func1()
