@@ -19,7 +19,7 @@ const size_t index_not_found = std::numeric_limits<size_t>::max();
 const size_t unspecified = std::numeric_limits<size_t>::max()-1;
 
 xml_token_attr_t::xml_token_attr_t() :
-    ns(XMLNS_UNKNOWN_ID), name(XML_UNKNOWN_TOKEN) {}
+    ns(XMLNS_UNKNOWN_ID), name(XML_UNKNOWN_TOKEN), transient(false) {}
 
 xml_token_attr_t::xml_token_attr_t(
     xmlns_id_t _ns, xml_token_t _name, const pstring& _value, bool _transient) :
